/*
# Author: Lionis de Souza Watanabe
*/
uniform sampler1D TRANSFERFUNCTION;
uniform sampler3D IMAGE3D;
uniform sampler3D CONTROLVOLUME;
uniform vec3 cursorPosition;
uniform float stepSize;
uniform bool draw3DCrosshair;
uniform float depth_slicing;

void main(){
	int markup;
	float scalar, depth;
	vec4 srcColor, dstColor, markupColor, controlTag;
	vec3 geomPos, cursorColor;

	dstColor = vec4(0.0);//initialize color
	cursorColor = vec3(0.8, 0.8, 0.8);
	geomPos = gl_TexCoord[1].xyz;

	controlTag = texture3D(CONTROLVOLUME, geomPos.xyz);
	markup = int(controlTag.x * float(INT_FORMAT_BITS));
	
	//all markups (except transparency) are considered valid voxels
	//to be included in visualization
	switch (markup){
	case TAG001://transparent
		break;
	case TAG002://painted
		dstColor = vec4(0.0, 1.0, 1.0, 1.0);
		break;
	case TAG003://point of reference
		dstColor = vec4(1.0, 0.0, 1.0, 1.0);
		break;
	case TAG004://point of interest
		dstColor = vec4(1.0, 1.0, 0.0, 1.0);
		break;
	case TAG005://line
		dstColor = vec4(1.0, 1.0, 1.0, 1.0);
		break;
	default: //if there wasn't a markup tag, begins depth processing 
		depth = controlTag.y;//get voxel's depth
		
		//1) all untagged voxels have depth=0 and must be shown.
		//2) also, voxels deeper (with greater depth) than the slicing mesh have to be shown.
		if(depth == 0.0 || depth_slicing <= depth){
			//(3.1) access data value at current position
			scalar = texture3D(IMAGE3D, geomPos.xyz).r; //scalar value
			
			//(3.2) apply transfer function
			srcColor  = texture1D(TRANSFERFUNCTION, scalar);
			srcColor *= srcColor.a; //Compute associated color of voxel
			
			//linear interpolation of dstColor and srcColor
			//opacity is just accumulated
			dstColor.rgb += mix(vec3(0.0), srcColor.rgb, 1.0 - dstColor.a);
			dstColor.a += srcColor.a;
		}
		break;
	}
	
	//draws 3D crosshair
	if (draw3DCrosshair){
		if (geomPos.x > cursorPosition.y && geomPos.x < cursorPosition.y + stepSize){
			dstColor.r += cursorColor.r;
		}
		if (geomPos.y > cursorPosition.z && geomPos.y < cursorPosition.z + stepSize) {
			dstColor.g += cursorColor.g;
		}
		if (geomPos.z > cursorPosition.x && geomPos.z < cursorPosition.x + stepSize) {
			dstColor.b += cursorColor.b;
		}
	}

	gl_FragColor = dstColor;
}